using System;
using System.Web;
using System.Web.SessionState;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;


namespace captcha
{

	/// <summary>
	/// Implementacia rozhrania <see cref="IHttpHandler"/>, ktory posiela
	/// obrazok s CAPTCHA kodom.
	/// </summary>
	public class CaptchaHttpHandler : IHttpHandler, IReadOnlySessionState
	{
		/// <summary>
		/// Kluc pod akym sa uklada do sessiony kod pre CAPTCHA
		/// </summary>
		internal const string SESSION_CaptchaKey = "CAPTCHAimg";

		// ------------------------------------------------------------------
		/// <summary>
		/// Override the IsReusable property 
		/// </summary>
		public bool IsReusable 
		{
			get 
			{
				return true; 
			}
		}

		private const int DEFAULT_WIDTH = 120;

		private const int DEFAULT_HEIGHT = 50;

		private string familyName = System.Drawing.FontFamily.GenericSerif.Name;
		
		/// <summary>
		/// For generating random numbers.
		/// </summary>
		private Random random = new Random();

		// ------------------------------------------------------------------
		/// <summary>
		/// Vygenerujeme obrazok a posleme ho ako response
		/// </summary>
		/// <param name="context"></param>
		public void ProcessRequest(HttpContext context)
		{

			// nacitame text, ktory budeme zobrazovat
			string textToDisplay = String.Empty;
			if (context.Session == null) 
			{
				textToDisplay = "nos";
			} 
			else 
			{
				textToDisplay = context.Session[ CaptchaHttpHandler.SESSION_CaptchaKey ] as string;
				if (textToDisplay == null) 
				{
					textToDisplay = "err";
				}
			}

			// vytvorime bitmapu
			Bitmap newBitmap = new Bitmap(DEFAULT_WIDTH, DEFAULT_HEIGHT, PixelFormat.Format32bppRgb);
			using (Graphics g = Graphics.FromImage(newBitmap))
			{
				// vyfarbime pozade
				g.FillRectangle(this.UsedBrush, this.UsedRectangle);
			
				// zapneme vyhladzovanie textu
				g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
		
				// vykrestlime text do obrzku
				
				// Nastavime format textu
				StringFormat format = new StringFormat();
				format.Alignment = StringAlignment.Center;
				format.LineAlignment = StringAlignment.Center;

				// Vytvorime cestu a nahodne ju 'pokrutime'
				GraphicsPath path = new GraphicsPath();
				path.AddString(textToDisplay, this.UsedFont.FontFamily, (int) this.UsedFont.Style, this.UsedFont.Size, this.UsedRectangle, format);
				float v = 4F;
				PointF[] points =
				{
					new PointF(this.random.Next(this.UsedRectangle.Width) / v, this.random.Next(this.UsedRectangle.Height) / v),
					new PointF(this.UsedRectangle.Width - this.random.Next(this.UsedRectangle.Width) / v, this.random.Next(this.UsedRectangle.Height) / v),
					new PointF(this.random.Next(this.UsedRectangle.Width) / v, this.UsedRectangle.Height - this.random.Next(this.UsedRectangle.Height) / v),
					new PointF(this.UsedRectangle.Width - this.random.Next(this.UsedRectangle.Width) / v, this.UsedRectangle.Height - this.random.Next(this.UsedRectangle.Height) / v)
				};
				Matrix matrix = new Matrix();
				path.Warp(points, this.UsedRectangle, matrix, WarpMode.Perspective, 0F);

				// Vykreslime text
				Brush hatchBrush = new HatchBrush(HatchStyle.LargeConfetti, Color.LightGray, Color.DarkGray);
				g.FillPath(hatchBrush, path);
			}	// Graphics - Dispose

			// nastavime content typ
			context.Response.ContentType = "image/jpeg";
			// zakazeme cachovanie
			context.Response.Cache.SetCacheability( HttpCacheability.NoCache );
			// Posleme obrazok
			newBitmap.Save(context.Response.OutputStream, ImageFormat.Jpeg);
		}

		/// <summary>
		/// Pouzity font
		/// </summary>
		public Font UsedFont
		{
			get 
			{
				if (this._usedFont == null) 
				{
					this._usedFont = new Font(System.Drawing.FontFamily.GenericSerif.Name, DEFAULT_HEIGHT*0.8F, FontStyle.Regular, GraphicsUnit.Pixel);
					System.Diagnostics.Debug.WriteLine("font cache NOT hit");
				}
				else 
				{
					System.Diagnostics.Debug.WriteLine("font cache hit");
				}
				return this._usedFont;
			}
		}

		private Font _usedFont = null;

		/// <summary>
		/// Pouzity obdlznik (rozmer obrazka)
		/// </summary>
		private Rectangle UsedRectangle = new Rectangle(0, 0, DEFAULT_WIDTH, DEFAULT_HEIGHT);

		/// <summary>
		/// Pouzity stetec pre pozadie obrazka
		/// </summary>
		public Brush UsedBrush
		{
			get 
			{
				if (this._usedBrush == null) 
				{
					this._usedBrush = new HatchBrush(HatchStyle.SmallConfetti, Color.LightGray, Color.White);
				}
				return this._usedBrush;
			}
		}

		private Brush _usedBrush = null;


	} // public class

} // namespace
